from vrml_analyse import *
from vrml_shape import *

from Blender import *

class vrml_0(vrml_analyse):

	def __init__(self,string,parent):
		self.VRMLReserved= ["Anchor","Billboard","Collision","Group","Inline", "LOD","Switch","Transform", "WorldInfo", "NavigationInfo"]
		self.keys = []
		self.string = string
		self.parent = parent


	def switchFunctions(self,string, parent):
		for key in self.keys:
			if key[0] == "Anchor":
				self.setAnchor(string[key[1]:key[2]])
			elif key[0] == "Billboard":
				self.setBillboard(string[key[1]:key[2]])
			elif key[0] == "Collision":
				self.setCollision(string[key[1]:key[2]])
			elif key[0] == "Group":
				group = vrml_group(string[key[1]:key[2]], parent)
				group.setGroup()
			elif key[0] == "Inline":
				self.setInline(string[key[1]:key[2]])
			elif key[0] == "LOD":
				self.setLOD(string[key[1]:key[2]])
			elif key[0] == "Switch":
				switch = vrml_switch(string[key[1]:key[2]], parent)
				switch.setSwitch()
			elif key[0] == "Transform":
				transform = vrml_transform(string[key[1]:key[2]], parent)
				transform.setTransform()
			elif key[0] == "WorldInfo":
				pass #self.setWorldInfo(string[key[1]:key[2]])
			elif key[0] == "NavigationInfo":
				pass #self.setNavigationInfo(string[key[1]:key[2]])

class vrml_groups(vrml_analyse):
	def addChildren(self,string,parent):
		pass
	def removeChildren(self,string, parent):
		pass
	def children(self,string,parent):
		'''processes the children nodes'''
		#print 'children'
		#print '##',string[:50], '\n\n', string[-10:], '##'
		for c in string:
			if c =='[':
				string = self.splittString(string)[1][1:-1]		#take the bracket string without the brackets
				break
			if c =='{':
				try: #string = self.deleteString(string, 'children')
					string = string[(string.find('children')+8):]
				except: pass
				break
		keys = self.getKeyList(string)
		self.switchFunctions(string, parent, keys)

	def bboxCenter(self,string,parent):
		pass
	def bboxSize(self,string,parent):
		pass

	def switchFunctions(self,string, parent, keys = None):
		if keys == None:
			keys = self.keys
		for key in keys:
			if key[0] == "Shape":
				shape = vrml_shape(string[key[1]:key[2]], parent)
				shape.setShape()
			elif key[0] == "children":
				self.children(string[key[1]:key[2]], parent)
			elif key[0] == "Transform":
				transform = vrml_transform(string[key[1]:key[2]], parent)
				transform.setTransform()


class vrml_group(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["addChildren", "removeChildren", "children", "bboxCenter", "bboxSize", 'Transform', 'Viewpoint']
		self.keys = []
		self.string = string
		self.parent = parent


	def setGroup(self):
		print 'creating group'
		'''processes everything within a switch node'''
		self.string = self.splittString(self.string)[1][1:-1]		#get the brace part
		self.keys = self.getKeyList(self.string)								#get the keys within the braces
		#print self.keys
		self.switchFunctions(self.string, self.parent)


class vrml_transform(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["addChildren", "removeChildren", "children", "bboxCenter", "bboxSize", "center", "rotation", "scale","scaleOrientation", "translation"]
		self.keys = []
		self.string = string
		self.parent = parent
		self.ob = None


	def makeBlenderName(self, name):
		'''Blender only takes names with a length of 21 characters
		but vrml and the blender python objects take longer name
		that will be different in blender.
		to avoid problems with truncated names this function shortens
		the name'''
		if len(name) < 16:
			return name
		name = name[:16]
		app = 0
		while 1:
			try:
				Object.Get(name+str(app))
				app += 1
			except:
				return name+str(app)
		return name


	def setTransform(self):
		'''creates a new Blender object and applies the given geometry,
		children and materials'''
		#print 'creating Object transformation'
		self.string = self.splittString(self.string)
		breaker = 0
		namestring = self.string[0]
		while 1:
			name = self.getName(namestring)
			if name == 'DEF':
				breaker = 1
			elif name == 'USE':
				breaker = -1
			elif breaker == -1:
				try:
					self.ob = Object.Get(getNewName(name))
				except: print 'object not found!'
			elif breaker == 1:
				#try:
					#self.ob = Object.Get(getNewName(name))
				#except:
				self.ob = Object.New('Mesh',self.makeBlenderName(name))
				setNewName(name,self.ob.getName())
				break
			elif name == 'Transform':
				self.ob = Object.New('Mesh')
				break
			namestring = self.deleteString(namestring, name)
		scene = Scene.GetCurrent()
		#try:
			#scene.link(self.ob)
		#except: pass
		self.string = self.string[1][1:-1]
		self.keys = self.getKeyList(self.string)								#get the keys within the braces
		keylist = []													#new,sorted keylist
		for key in self.keys:							#put the geometries in the keylist first
			if key[0] == 'children':							#necessary because the translation must be applied to that geometry
				keylist.append(key)
		for key in self.keys:							#put the geometries in the keylist first
			if key[0] != 'children':							#necessary because the translation must be applied to that geometry
				keylist.append(key)
		self.keys = keylist								#make the old keylist the new :-)
		self.VRMLReserved.append('Shape')
		self.VRMLReserved.append('Transform')
		self.switchFunctions(self.string, self.ob.getName())
		#try:
			#self.ob = Object.Get(getNewName(obname))
		#except:
			#self.ob = Object.New('Mesh',obname)
			#setNewName(obname,self.ob.getName())
		if self.parent != None:
			try:
				pob = Object.Get(getNewName(self.parent))
				pob.makeParent([self.ob])
			except: pass
		Redraw()
		#self.start(self.string[1][1:-1],name)


	def switchFunctions(self,string, parent, keys = None):
		if keys == None:
			keys = self.keys
		for key in keys:
			if key[0] == "addChildren":
				self.addChildren(string[key[1]:key[2]],parent)
			elif key[0] == "removeChildren":
				self.removeChildren(string[key[1]:key[2]],parent)
			elif key[0] == "children":
				self.children(string[key[1]:key[2]], parent)
			elif key[0] == "bboxCenter":
				self.bboxCenter(string[key[1]:key[2]])
			elif key[0] == "bboxSize":
				self.bboxSize(string[key[1]:key[2]])
			elif key[0] == "center":
				pass #self.center(string[key[1]:key[2]])
			elif key[0] == "rotation":
				self.rotation(string[key[1]:key[2]])
			elif key[0] == "scale":
				self.scale(string[key[1]:key[2]])
			elif key[0] == "scaleOrientation":
				self.scaleOrientation(string[key[1]:key[2]])
			elif key[0] == "translation":
				self.translation(string[key[1]:key[2]])
			elif key[0] == "Shape":
				shape = vrml_shape(string[key[1]:key[2]], parent)
				shape.setShape()
			elif key[0] == "Group":
				pass
			elif key[0] == "Transform":
				transform = vrml_transform(string[key[1]:key[2]], parent)
				transform.setTransform()

	def translation(self, string):
		#print 'performing translation'
		while 1:
			name = self.getName(string)
			if name == None:
				#print 'no translation data found!'
				return
			if name == 'translation':
				translation = self.getValues(string,3)
				#print 'translation',translation
				self.ob.setLocation(float(translation[0]),float(translation[1]),float(translation[2]))
				return
			string = self.deleteString(string, name)


	def rotation(self, string):
		#print 'performing rotation'
		while 1:
			name = self.getName(string)
			if name == None:
				#print 'no rotation data found!'
				return
			if name == 'rotation':
				rotation = self.getValues(string,4)
				#print 'rotation',rotation
				#raw_input()
				self.ob.setEuler(float(rotation[0])*float(rotation[3]),float(rotation[1])*float(rotation[3]),float(rotation[2])*float(rotation[3]))
				return
			string = self.deleteString(string, name)


	def scale(self, string):
		while 1:
			name = self.getName(string)
			if name == None:
				print 'no scale data found!'
				return
			elif name == 'scale':
				scale = self.getValues(string,3)
				#me = self.ob.GetData()
				self.ob.SizeX *= float(scale[0])
				self.ob.SizeY *= float(scale[1])
				self.ob.SizeZ *= float(scale[2])
				#for v in me.verts:
					#v.co[0] *= float(scale[0])
					#v.co[1] *= float(scale[1])
					#v.co[2] *= float(scale[2])
				#me.update()
				return
			string = self.deleteString(string, name)

	def scaleOrientation(self, string):
		return



class vrml_billboard(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["addChildren", "removeChildren", "children", "bboxCenter", "bboxSize", "axisOfRotation"]
		self.keys = []
		self.string = string
		self.parent = parent

class vrml_collision(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["addChildren", "removeChildren", "children", "bboxCenter", "bboxSize", "collide", "proxy", "collideTime"]
		self.keys = []
		self.string = string
		self.parent = parent

class vrml_switch(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["choice", "whichChoice", "Shape"]
		self.keys = []
		self.string = string
		self.parent = parent

	def setSwitch(self):
		print 'creating switch'
		'''processes everything within a switch node'''
		self.string = self.splittString(self.string)[1][1:-1]		#get the brace part
		self.keys = self.getKeyList(self.string)								#get the keys within the braces
		#print self.keys
		self.switchFunctions(self.string, self.parent)


	def choice(self, string, parent):
		string = self.splittString(string)[1][1:-1]		#get the brace part
		self.keys = self.getKeyList(string)
		self.switchFunctions(string, self.parent)

	def whichChoice(self, string, parent):
		pass

	def switchFunctions(self,string, parent, keys = None):
		if keys == None:
			keys = self.keys
		for key in keys:
			if key[0] == "choice":
				self.choice(string[key[1]:key[2]],parent)
			elif key[0] == "whichChoice":
				self.whichChoice(string[key[1]:key[2]],parent)
			elif key[0] == "Shape":
				shape = vrml_shape(string[key[1]:key[2]], parent)
				shape.setShape()

class vrml_anchor(vrml_groups):

	def __init__(self,string,parent):
		self.VRMLReserved= ["addChildren", "removeChildren", "children", "bboxCenter", "bboxSize", "description", "parameter", "url"]
		self.keys = []
		self.string = string
		self.parent = parent
