from vrml_analyse import *
from vrml_aux import *
from Blender import *
import sys

class vrml_shape(vrml_analyse):

	def __init__(self,string,parent):
		self.VRMLReserved= ["appearance", "geometry"]
		self.keys = []
		self.string = string
		self.parent = parent

	def setShape(self):
		'''sorts the material and geometry nodes and starts the corresponding functions'''
		#print '###'+self.string[:50]+'\n'+self.string[-50:]+'###'
		string = self.splittString(self.string)[1][1:-1]		#get the brace part
		self.keys = self.getKeyList(string)								#get the keys within the braces
		keylist = []													#new,sorted keylist
		for key in self.keys:							#put the geometries in the keylist first
			if key[0] == 'geometry':							#necessary because the material must be applied to that geometry
				keylist.append(key)								#if tere is no geometry, no material can be applied
		for key in self.keys:							#now add the appearances
			if key[0] == 'appearance':
				keylist.append(key)
		self.keys = keylist								#make the old keylist the new :-)
		#for i in self.keys:
			#print i[0]
		self.switchFunctions(string, self.parent)


	def switchFunctions(self,string, parent):
		for key in self.keys:
			if key[0] == "geometry":
				geometry = vrml_geometry(string[key[1]:key[2]], parent)
				geometry.setGeometry()
			elif key[0] == "appearance":
				appearance = vrml_appearance(string[key[1]:key[2]], parent)
				appearance.setAppearance()
			else:
				print 'key not found', key[0], 'in vrml_shape'

class vrml_geometry(vrml_analyse):
	def __init__(self,string,parent):
		self.VRMLReserved= ['Box','Cone','Cylinder','Sphere','PointSet', 'IndexedFaceSet', 'IndexedLineSet','ElevationGrid','Extrusion', 'color', 'coord', 'set_colorIndex', 'set_coordIndex', 'colorIndex', 'colorPerVertex', 'coordIndex', 'set_normalIndex', 'set_texCoordIndex', 'normal', 'texCoord', 'ccw', 'colorIndex', 'colorPerVertex', 'convex', 'creaseAngle', 'normalIndes', 'normalPerVertex', 'solid', 'texCoordIndex', 'set_height', 'height', 'xDimension', 'xSpacing', 'zDimension', 'zSpacing', 'set_crossSection', 'set_orientation', 'set_scale', 'set_spine', 'beginCap', 'crossSection', 'endCap', 'orientation', 'scale', 'spine', 'point', 'vector', 'Color', 'textureCoordinate']
		self.keys = []
		self.string = string
		self.parent = parent



	def makeBlenderName(self,name):
		'''Blender only takes names with a length of 21 characters
		but vrml and the blender python objects take longer name
		that will be different in blender.
		to avoid problems with truncated names this function shortens
		the name'''
		if len(name) < 16:
			return name
		name = name[:16]
		app = 0
		while 1:
			#print name+str(app)
			me = NMesh.GetRaw(name+str(app))
			#print me
			#raw_input()
			if me != None:
				app += 1
			else:
				return name+str(app)
		return name
	
	
	def mergeGeometries(self, geo1, geo2):
		'''It's not Possible to link more than one Mesh to a Mesh object
		but in VRML it is possible, so we have to merge the to geometries to one'''
		for face in geo2.faces:	#the faces of the second geometry
			newface = []
			for i in face:				#the vertices of the face
				vert = i #geo2.verts[i]
				pos = 0
				index = -1
				for v in geo1.verts:		#check if the vertice is already in the vertex list of the first geometry
					if v == vert:
						index = pos
				if index > -1:
					newface.append(index)
				else:
					newface.append(len(geo1.verts))
					geo1.verts.append(vert)
			geo1.faces.append(face)		#new face is ready
			geo1.faces[-1].uv = face.uv		#add the uv coordinates to the face
		return geo1


	def setGeometry(self):
		'''creates a new Blender object and applies the given geometry,
		children and materials'''
		print 'creating geometry...'
		self.string = self.splittString(self.string)
		breaker = 0
		namestring = self.string[0]
		while 1:
			name = self.getName(namestring)
			if name == None:
				print 'no geometry built!'
				return
			elif name == 'DEF':
				breaker = 1
			elif name == 'USE':
				breaker = -1
			elif breaker == -1:
				ob = Object.Get(getNewName(self.parent))
				if ob.getData().name == None:
					self.me = NMesh.GetRaw(getNewName(name))
					ob.link(self.me)
				else:
					self.me = self.mergeGeometries(ob.getData(),NMesh.GetRaw(getNewName(name)))
				#print 'Meshname',self.me.name
				#print 'objectname',self.parent
				#print 'getnewname', getNewName(self.parent)
				#print 'linking geometry',self.me.name,'to', ob.getName()
				self.me.update()
				return
			elif name == 'Box' or name == 'Cone' or name == 'Cylinder' or name == 'Sphere' or name == 'PointSet' or name == 'IndexedFaceSet' or name == 'IndexedLineSet' or name == 'ElevationGrid' or name == 'Extrusion':
				if breaker < 2:
					self.me = NMesh.New()
					setNewName(name,self.me.name)
				break
			elif breaker == 1:
				self.me = NMesh.New(self.makeBlenderName(name))
				#ob = Object.New('Mesh')
				#self.me = ob.getData()
				#self.me.name = name
				#self.me.update()
				#self.me = ob.getData()
				setNewName(name,self.me.name)
				#self.me = NMesh.New(name)
				breaker = 2
			namestring = self.deleteString(namestring, name)
		self.start(namestring+self.string[1],self.me.name)

		#print 'meshupdate in set geometry', self.me.name
		self.me.update()
		if self.parent != None:
			ob = Object.Get(getNewName(self.parent))
			ob.link(self.me)
			#print 'object',ob.getName(),'linked to', ob.getData().name,'bzw',self.me.name


	def Box(self,string,parent):
		self.me = NMesh.New('Cube',parent)
		while 1:
			name = self.getName(string)
			if name == None:
				break
			if name == 'size':
				size = self.getValues(string,3)
				for v in self.me.verts:
					v.co[0] *= size[0]/2
					v.co[1] *= size[1]/2
					v.co[2] *= size[2]/2
				return
			string = self.deleteString(string, name)


	def Sphere(self,string,parent):
		#print 'creating a sphere', string, parent
		self.me = NMesh.New(parent)
		while 1:
			name = self.getName(string)
			if name == None:
				break
			if name == 'radius':
				size = self.getValues(string,1)[0]
				for v in self.me.verts:
					v.co[0] *= size/2
					v.co[1] *= size/2
					v.co[2] *= size/2
				return
			string = self.deleteString(string, name)

	def Cone(self,string,parent):
		pass

	def Cylinder(self,string,parent):
		pass

	def PointSet(self,string,parent):
		pass

	def ElevationGrid(self,string,parent):
		pass

	def Extrusion(self,string,parent):
		pass


	def IndexedFaceSet(self, string, parent):
		#print 'creating indexed face...'
		string = self.splittString(string)[1][1:-1]
		keys = self.getKeyList(string)
		keylist = []													#new,sorted keylist
		for key in keys:
			if key[0] == 'coord':								#add coords first
				keylist.append(key)
				break
		for key in keys:
			if key[0] == 'coordIndex':				#and now the faces
				keylist.append(key)								#maybe further sortings are necessary, but later...:-)
				break
		for key in keys:
			if key[0] == 'texCoord':								#add coords first
				keylist.append(key)
				break
		for key in keys:
			if key[0] == 'texCoordIndex':				#and now the faces
				keylist.append(key)								#maybe further sortings are necessary, but later...:-)
				break
		for key in keys:							#now add all the other nodes
			if key[0] != 'coord' and key[0] != 'coordIndex' and key[0] != 'texCoord' and key[0] != 'texCoordIndex':
				keylist.append(key)
		#print keylist
		self.switchFunctions(string, parent, keylist)

	def coord(self, string, parent):
		#print string[:50], ' ###', string[:50]
		#string = self.splittString(string)[1][1:-1]
		#keys = self.getKeyList(string)
		#print 'coord keylist:', keys
		#self.switchFunctions(string, parent, keys)
		string = self.splittString(string)
		breaker = 0
		namestring = string[0]
		#print 'namestring:',namestring
		while 1:
			name = self.getName(namestring)
			#print 'name',name
			if name == None:
				print 'no points defined!'
				return
			elif name == 'DEF':
				breaker = 1
			elif name == 'USE':
				breaker = -1
			elif breaker == -1:
				#print 'use found'
				#print 'name',name
				print 'using points from:', getNewName(name)
				me2 = NMesh.GetRaw(getNewName(name))
				print 'name of the used mesh (points)',me2.name
				for v in me2.verts:
					#print v
					self.me.verts.append(v)
				print 'coordinates applied'
				#self.me.verts = me2.verts
				#self.me.update()
				return
			elif breaker == 1:
				setNewName(name,parent)
				breaker = 2
			elif name == 'Coordinate':
				break
			namestring = self.deleteString(namestring, name)
		keys = self.getKeyList(string[1][1:-1])
		self.switchFunctions(string[1][1:-1], parent, keys)

	def point(self, string, parent):
		#print 'creating points'
		#me = NMesh.GetRaw(parent)
		tripel = []
		count = 0
		float_char = '-1234567890.e'
		floatret = ''
		found = -1
		for n in string:
			if  float_char.find(n)> -1:
				floatret = floatret+n
				found = 1
			if found == 1 and float_char.find(n)<0:
				#print floatret
				tripel.append(float(floatret))
				count += 1
				found = -1
				floatret = ''
			if count > 2:
				#if parent == 'head-FACES' or parent == 'head':
					#print tripel
					#raw_input()
				v = NMesh.Vert(tripel[0], tripel[1], tripel[2]) # create the NMVert
				self.me.verts.append(v)
				tripel = []
				count = 0
		self.me.update()

	def normal(self, string, parent):
		pass

	def convex(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'convex':
				string = self.deleteString(string, name)
				name = self.getName(string)
				if name == 'FALSE':
					pass	#set Mesh to not convex
				elif name == 'TRUE':
					pass	#set Mesh to convex
				return
			string = self.deleteString(string, name)


	def colorPerVertex(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'colorPerVertex':
				string = self.deleteString(string, name)
				name = self.getName(string)
				if name == 'FALSE':
					pass	#set Mesh to not colorPerVertex
				elif name == 'TRUE':
					pass	#set Mesh to colorPerVertex
				return
			string = self.deleteString(string, name)

	def normalPerVertex(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'normalPerVertex':
				string = self.deleteString(string, name)
				name = self.getName(string)
				if name == 'FALSE':
					pass	#set Mesh to not normalPerVertex
				elif name == 'TRUE':
					pass	#set Mesh to normalPerVertex
				return
			string = self.deleteString(string, name)

	def solid(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'solid':
				string = self.deleteString(string, name)
				name = self.getName(string)
				if name == 'FALSE':
					pass	#set Mesh to not solid
				elif name == 'TRUE':
					pass	#set Mesh to solid
				return
			string = self.deleteString(string, name)

	def ccw(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'ccw':
				string = self.deleteString(string, name)
				name = self.getName(string)
				if name == 'FALSE':
					pass	#set Mesh to not ccw
				elif name == 'TRUE':
					pass	#set Mesh to ccw
				return
			string = self.deleteString(string, name)


	def coordIndex(self, string, parent):
		#print 'creating faces'
		#me = NMesh.GetRaw(parent)
		face = []
		count = 0
		int_char = '-1234567890'
		intret = ''
		found = -1
		for n in string:
			if  int_char.find(n)> -1:
				intret = intret+n
				found = 1
			if found == 1 and int_char.find(n)<0:
				if int(intret) <0:
					face = NMesh.Face(face)
					self.me.faces.append(face)
					face = []
				else:
					self.me.name
					try:
						face.append(self.me.verts[int(intret)])
					except:
						print 'face could not be appended'
				count += 1
				found = -1
				intret = ''
		#print 'mesh in coord index', self.me.name
		self.me.update()
		###It is possible, that the last face is missing, because it has not endet with a -1, checking not implemented yet



	def texCoord(self, string, parent):
		'''uses normal branch, dont know why'''
		#print 'applying uv coordinates'
		string = self.splittString(string)[1][1:-1]
		UVList = []
		tupel = []
		count = 0
		float_char = '-1234567890.e'
		floatret = ''
		found = -1
		for n in string:
			if  float_char.find(n)> -1:
				#print 'float_char found', n
				floatret = floatret+n
				found = 1
			if found == 1 and float_char.find(n)<0:
				tupel.append(float(floatret))
				count += 1
				found = -1
				floatret = ''
			if count > 1:
				#print 'setting Point at', tripel
				UVList.append((tupel[0],tupel[1]))
				tupel = []
				count = 0
		#print UVList
		return UVList
		#for coords in UVList:
			#try: self.me.faces[count].uv = coords
			#except: break
			#print self.me.faces[count].uv
			#count += 1
			#print 'coordinates applied'
			#self.me.update()
			#print 'mesh updated'


	def texCoordIndex(self, string, parent):
		'''doesnot really do anything, don't know if i need to use this info.... :-)
		correct implementation later...
		texCoord and texCoordIndex must return their lists und put together
		in switch functions...'''
		facelist = []
		face = []
		count = 0
		int_char = '-1234567890'
		intret = ''
		found = -1
		for n in string:
			if  int_char.find(n)> -1:
				intret = intret+n
				found = 1
			if found == 1 and int_char.find(n)<0:
				if int(intret) <0:
					facelist.append(face)
					face = []
				else:
					face.append(int(intret))
				count += 1
				found = -1
				intret = ''
		return facelist

	def applyUVs(self, tc, ti):				#tc == texCoords, ti == texCoordIndexs
		pos = 0
		for face in self.me.faces:
			uvs = []
			for num in ti[pos]:
				uvs.append(tc[num])
			face.uv = uvs
			pos += 1
		self.me.hasFaceUV(1)
		self.me.update(1)

	def color(self,string, parent):
		pass

	def switchFunctions(self,string, parent, keys = None):
		texCoords = None
		texCoordIndexs = None
		if keys == None:
			keys = self.keys
		for key in keys:
			if key[0] == "Box":
				self.Box(string[key[1]:key[2]],parent)
			elif key[0] == "Cone":
				self.Cone(string[key[1]:key[2]],parent)
			elif key[0] == "Cylinder":
				self.Cylinder(string[key[1]:key[2]], parent)
			elif key[0] == "Sphere":
				self.Sphere(string[key[1]:key[2]], parent)
			elif key[0] == "PointSet":
				self.PointSet(string[key[1]:key[2]], parent)
			elif key[0] == "IndexedFaceSet":
				self.IndexedFaceSet(string[key[1]:key[2]], parent)
			elif key[0] == "IndexedLineSet":
				self.IndexedLineSet(string[key[1]:key[2]], parent)
			elif key[0] == "ElevationGrid":
				self.ElevationGrid(string[key[1]:key[2]], parent)
			elif key[0] == "Extrusion":
				self.Extrusion(string[key[1]:key[2]], parent)
			elif key[0] == "color":
				self.color(string[key[1]:key[2]], parent)
			elif key[0] == "coord":
				self.coord(string[key[1]:key[2]], parent)
			elif key[0] == "coordIndex":
				self.coordIndex(string[key[1]:key[2]], parent)
			elif key[0] == "texCoord":
				texCoords = self.texCoord(string[key[1]:key[2]], parent)
			elif key[0] == "normal":
				self.normal(string[key[1]:key[2]], parent)
			elif key[0] == "convex":
				self.convex(string[key[1]:key[2]], parent)
			elif key[0] == "normalPerVertex":
				self.normalPerVertex(string[key[1]:key[2]], parent)
			elif key[0] == "solid":
				self.solid(string[key[1]:key[2]], parent)
			elif key[0] == "texCoordIndex":
				texCoordIndexs = self.texCoordIndex(string[key[1]:key[2]], parent)
			elif key[0] == "point":
				self.point(string[key[1]:key[2]], parent)
			elif key[0] == "vector":
				self.vector(string[key[1]:key[2]], parent)
		if texCoords != None and texCoordIndexs != None:
			self.applyUVs(texCoords,texCoordIndexs)
'''
'set_colorIndex', 'set_coordIndex', 'colorIndex', 'colorPerVertex', 'set_normalIndex', 'set_texCoordIndex', 'ccw', 'colorIndex', 'colorPerVertex',  'creaseAngle', 'normalIndes',  'set_height', 'height', 'xDimension', 'xSpacing', 'zDimension', 'zSpacing', 'set_crossSection', 'set_orientation', 'set_scale', 'set_spine', 'beginCap', 'crossSection', 'endCap', 'orientation', 'scale', 'spine', 'vector', 'Normal', 'Color', 'textureCoordinate'		# still missing
'''
class vrml_appearance(vrml_analyse):

	def __init__(self,string,parent):
		self.VRMLReserved= ["material", "texture", "textureTransform", 'diffuseColor', 'specularColor', 'emissiveColor', 'shininess', 'transparency','ambientIntensity']
		self.keys = []
		self.string = string
		self.parent = parent



	def switchFunctions(self,string, parent, keys = None):
		if keys == None:
			keys = self.keys
		for key in keys:
			if key[0] == "material":
				self.material(string[key[1]:key[2]],parent)
			elif key[0] == "texture":
				self.texture(string[key[1]:key[2]],parent)
			elif key[0] == "textureTransform":
				self.textureTransform(string[key[1]:key[2]], parent)
			elif key[0] == "diffuseColor":
				self.diffuseColor(string[key[1]:key[2]], parent)
			elif key[0] == "specularColor":
				self.specularColor(string[key[1]:key[2]], parent)
			elif key[0] == "emissiveColor":
				self.emissiveColor(string[key[1]:key[2]], parent)
			elif key[0] == "shininess":
				self.shininess(string[key[1]:key[2]], parent)
			elif key[0] == "transparency":
				self.transparency(string[key[1]:key[2]], parent)
			elif key[0] == "ambientIntensity":
				self.ambientIntensity(string[key[1]:key[2]], parent)

	def setAppearance(self):
		'''defines materials and textures for the object given in parent'''
		string = self.splittString(self.string)[1][1:-1]
		keys = self.getKeyList(string)
		self.switchFunctions(string, self.parent,keys)

	def material(self,string, parent):
		'''creates a new Blender material and applies the given properties'''
		print 'creating Material...'
		string = self.splittString(string)
		breaker = 0
		namestring = string[0]
		while 1:
			name = self.getName(namestring)
			if name == None:
				print 'no material defined!'
				return
			if name == 'DEF':
				breaker = 1
			elif name == 'USE':
				breaker = -1
			elif breaker == -1:
				#print 'object in materials:',parent
				ob = Object.Get(parent)
				me = ob.getData()
				self.ma = Material.Get(name)
				me.addMaterial(self.ma)
				#print 'mesh in materials', me.name, breaker
				#print 'material', self.ma.name, 'linked to', me.name
				me.update()
				return
			elif breaker == 1:
				try:
					self.ma = Material.Get(name)
				except:
					self.ma = Material.New(name)
				breaker = 2
			elif name == 'Material':
				if breaker < 2:
					self.ma = Material.New()
				break
			namestring = self.deleteString(namestring, name)
			#print 'name:',name
			#print 'breaker:',breaker
		keys = self.getKeyList(string[1][1:-1])
		self.switchFunctions(string, name,keys)
		if parent != None:
				ob = Object.Get(parent)
				print 'object to link to', ob.getName()
				me = ob.getData()
				print 'Mesh to link to',me.name
				me.addMaterial(self.ma)
				#print 'mesh in materials', me.name, breaker
				me.update()


	def diffuseColor(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'diffuseColor':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,3)
				self.ma.rgbCol = floats
				return
			string = self.deleteString(string, name)

	def specularColor(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'specularColor':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,3)
				self.ma.setSpecCol(floats)
				return
			string = self.deleteString(string, name)

	def emissiveColor(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'emissiveColor':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,3)
				self.ma.setEmit(averageValue(floats))
				return
			string = self.deleteString(string, name)

	def shininess(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'shininess':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,1)
				self.ma.setHardness(int(floats[0]*255))
				return
			string = self.deleteString(string, name)

	def transparency(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'transparency':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,1)
				self.ma.setAlpha(1-floats[0])
				return
			string = self.deleteString(string, name)

	def ambientIntensity(self, string, parent):
		while 1:
			name = self.getName(string)
			if name == None:
				return
			if name == 'ambientIntensity':
				string = self.deleteString(string, name)
				name = self.getName(string)
				floats = self.getValues(string,1)
				self.ma.setAmb(floats[0])
				return
			string = self.deleteString(string, name)


	def texture(self,string, parent):
		'''creates a new Blender material and applies the given properties'''
		#print 'creating texture'
		#print 'texture: ###',string, '##'
		string = self.splittString(string)
		breaker = 3
		namestring = string[0]
		while 1:
			name = self.getName(namestring)
			if name == None:
				print 'no texture defined!'
				return
			if breaker == 3:
				if name == 'texture':
					breaker = 0
			else:
				if name == 'DEF':
					breaker = 1
				elif name == 'USE':
					breaker = -1
				elif breaker == -1:
					ob = Object.Get(parent)
					me = ob.getData()
					self.te = Texture.Get(name)
					img = self.te.getImage()
					for face in me.faces:
						face.image = img
					print 'mesh in texture', me.name, breaker
					me.update()
					return
				elif breaker == 1:
					try:
						self.te = Texture.Get(name)
					except:
						self.te = Texture.New(name)
					breaker = 2
				elif name == 'ImageTexture':
					if breaker < 2:
						self.te = Texture.New()
					break
			namestring = self.deleteString(namestring, name)
			#print 'name',name
			#print 'breaker',breaker
		img = self.getImageTexture(string[1][1:-1])
		try:
			self.te.setImage(img)
			if parent != None:
				print 'linking texture to mesh'
				ob = Object.Get(parent)
				me = ob.getData()
			#me = NMesh.GetRaw(me.name)
			#print 'object',ob.getName(),'with mesh', me.name, 'linked to texture', img
				ma = me.materials[0]
			#print 'material loaded'
				self.te.setType('Image')
			#print 'type set to image'
				ma.setTexture(0,self.te)
			#print 'texture linked'
				me.hasFaceUV(1)
				me.mode = 516
			#me.smooth = 1
				for face in me.faces:
					face.image = img
				me.update(1)
				ma.setTexture(0, self.te)
			#try:
				tex = ma.getTextures()
				tex[0].texco = 16
				ma.setTexture(0, self.te)
			#self.te.flags = 1
			#self.te.mtex.texco = 16
			######self.te.texco = 16					#should set texture mapping to uv, but doesn't work
			#self.te.tex.texco = 'UV'
			#except:
				#print 'TexCo not set'
				#for face in me.faces:
					#face.image = img
		except:
			print "couldn't set Texture Image"

	def textureTransform(self, string, parent):
		pass

	def getImageTexture(self,string):
		img = None
		pos1 = string.find('"')
		pos2 = string[pos1+1:].find('"')+pos1+1
		image = string[pos1+1:pos2]
		#print image
		try:
			img = Image.Load(image) # load the image from disk
			print 'image loaded'
		except:
			print "couldn't load image"
		return img
